@@ -225,9 +225,9 @@ mod tests {
             array.validity(),
             &Some(Bitmap::from_u8_slice(&[0b00000101], 3))
         );
-        assert_eq!(array.is_valid(0), true);
-        assert_eq!(array.is_valid(1), false);
-        assert_eq!(array.is_valid(2), true);
+        assert!(array.is_valid(0));
+        assert!(!array.is_valid(1));
+        assert!(array.is_valid(2));
 
         let array2 = PrimitiveArray::<i32>::from_data(
             DataType::Int32,
