@@ -8,8 +8,9 @@ use crate::{
 };
 
 /// Trait describing any type that can be used to index a slot of an array.
-pub trait Index: NativeType {
+pub trait Index: NativeType + NaturalDataType {
     fn to_usize(&self) -> usize;
+    fn from_usize(index: usize) -> Option<Self>;
 }
 
 /// Trait describing types that can be used as offsets as per Arrow specification.
@@ -17,14 +18,12 @@ pub trait Index: NativeType {
 /// # Safety
 /// Do not implement.
 pub unsafe trait Offset:
-    Index + NaturalDataType + Num + Ord + std::ops::AddAssign + std::ops::Sub + num::CheckedAdd
+    Index + Num + Ord + std::ops::AddAssign + std::ops::Sub + num::CheckedAdd
 {
     fn is_large() -> bool;
 
     fn to_isize(&self) -> isize;
 
-    fn from_usize(value: usize) -> Option<Self>;
-
     fn from_isize(value: isize) -> Option<Self>;
 }
 
@@ -34,11 +33,6 @@ unsafe impl Offset for i32 {
         false
     }
 
-    #[inline]
-    fn from_usize(value: usize) -> Option<Self> {
-        Self::try_from(value).ok()
-    }
-
     #[inline]
     fn from_isize(value: isize) -> Option<Self> {
         Self::try_from(value).ok()
@@ -56,11 +50,6 @@ unsafe impl Offset for i64 {
         true
     }
 
-    #[inline]
-    fn from_usize(value: usize) -> Option<Self> {
-        Some(value as i64)
-    }
-
     #[inline]
     fn from_isize(value: isize) -> Option<Self> {
         Self::try_from(value).ok()
@@ -77,6 +66,11 @@ impl Index for i32 {
     fn to_usize(&self) -> usize {
         *self as usize
     }
+
+    #[inline]
+    fn from_usize(value: usize) -> Option<Self> {
+        Self::try_from(value).ok()
+    }
 }
 
 impl Index for i64 {
@@ -84,6 +78,11 @@ impl Index for i64 {
     fn to_usize(&self) -> usize {
         *self as usize
     }
+
+    #[inline]
+    fn from_usize(value: usize) -> Option<Self> {
+        Self::try_from(value).ok()
+    }
 }
 
 impl Index for u32 {
@@ -91,6 +90,11 @@ impl Index for u32 {
     fn to_usize(&self) -> usize {
         *self as usize
     }
+
+    #[inline]
+    fn from_usize(value: usize) -> Option<Self> {
+        Self::try_from(value).ok()
+    }
 }
 
 impl Index for u64 {
@@ -98,6 +102,11 @@ impl Index for u64 {
     fn to_usize(&self) -> usize {
         *self as usize
     }
+
+    #[inline]
+    fn from_usize(value: usize) -> Option<Self> {
+        Self::try_from(value).ok()
+    }
 }
 
 #[inline]
